package com.nanyan.securitylink.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.nanyan.securitylink.config.AppConfig;
import com.nanyan.securitylink.service.AlertService;
import com.nanyan.securitylink.service.TokenRouteMappingService;
import io.netty.channel.ChannelOption;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.client.reactive.ReactorClientHttpConnector;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;
import reactor.netty.http.client.HttpClient;

import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeoutException;

@Slf4j
@Component
public class FailoverGatewayFilterFactory extends AbstractGatewayFilterFactory<FailoverGatewayFilterFactory.Config> {

    private static final String CACHED_REQUEST_BODY_KEY = "cachedRequestBody";
    private static final String REQUEST_ID = "request_id";
    private final WebClient.Builder webClientBuilder;
    private final TokenRouteMappingService tokenRouteMappingService;
    // 添加错误统计
    private static final int WINDOW_SIZE_SECONDS = 100; // 1分钟窗口
    private static final double ERROR_THRESHOLD = 0.5; // 75%错误率阈值
    private static Long lastAlarmTime = System.currentTimeMillis();

    @Autowired
    AlertService alertService;
    @Autowired
    AppConfig appConfig;
    private final Map<String, CircularQueue> errorStatsMap = new ConcurrentHashMap<>();

    public FailoverGatewayFilterFactory(WebClient.Builder webClientBuilder,
                                        TokenRouteMappingService tokenRouteMappingService) {
        super(Config.class);
        this.webClientBuilder = webClientBuilder;
        this.tokenRouteMappingService = tokenRouteMappingService;
    }

    // 内部类：环形队列，用于记录请求结果
    private static class CircularQueue {
        AppConfig appConfig;
        private final long[] timestamps;
        private final boolean[] results; // true表示成功，false表示失败
        private int currentIndex = 0;
        private final int capacity;
        private final Object lock = new Object();

        public CircularQueue(int capacity, AppConfig appConfig) {
            this.capacity = capacity;
            this.timestamps = new long[capacity];
            this.results = new boolean[capacity];
            this.appConfig = appConfig;
        }

        public void add(boolean success) {
            synchronized (lock) {
                timestamps[currentIndex] = System.currentTimeMillis();
                results[currentIndex] = success;
                currentIndex = (currentIndex + 1) % capacity;
            }
        }

        public double getErrorRate() {
            synchronized (lock) {
                long currentTime = System.currentTimeMillis();
                long oneMinuteAgo = currentTime - appConfig.getWindowsTime();
                int totalRequests = 0;
                int failedRequests = 0;

                for (int i = 0; i < capacity; i++) {
                    if (timestamps[i] > oneMinuteAgo) {
                        totalRequests++;
                        if (!results[i]) {
                            failedRequests++;
                        }
                    }
                }
                log.info("failedRequests:{}, totalRequests:{}", failedRequests, totalRequests);
                return totalRequests == 0 ? 0 : (double) failedRequests / totalRequests;
            }
        }
    }

    private CircularQueue getOrCreateErrorStats(String primaryUrl) {
        return errorStatsMap.computeIfAbsent(primaryUrl,
                k -> new CircularQueue(WINDOW_SIZE_SECONDS * 10, appConfig)); // 存储10次/秒的采样
    }

    private boolean shouldSkipPrimary(String primaryUrl) {
        CircularQueue stats = getOrCreateErrorStats(primaryUrl);
        double errorRate = stats.getErrorRate();
        log.info("rate: {}%, url: {}", errorRate * 100, primaryUrl);
        boolean skip = errorRate >= ERROR_THRESHOLD;
        if (skip) {
            // 发送告警
            sendAlarmIfNeeded(primaryUrl, errorRate);
            log.warn("Skipping primary endpoint due to high error rate: {}%, url: {}",
                    errorRate * 100, primaryUrl);
        }
        return skip;
    }

    private final Map<String, Long> alarmSentMap = new ConcurrentHashMap<>();

    private void sendAlarmIfNeeded(String primaryUrl, double errorRate) {
        long currentTime = System.currentTimeMillis();
        Long lastAlarmTime = alarmSentMap.get(primaryUrl);

        // 如果从未发送过告警，或者距离上次告警已经超过冷却时间
        if (lastAlarmTime == null || (currentTime - lastAlarmTime) > 20 * 60 * 1000) {
            String alarmMessage = String.format(
                    "Service degradation detected: %s has high error rate (%.2f%%), switching to fallback service",
                    primaryUrl, errorRate * 100);

            try {
                alertService.sendAlertMsg("dify 异常，已经切换到JAVA服务");
                alarmSentMap.put(primaryUrl, currentTime);
                log.warn("Alarm sent: {}", alarmMessage);
            } catch (Exception e) {
                log.error("Failed to send alarm: {}", e.getMessage());
            }
        }
    }

    @Override
    public GatewayFilter apply(Config config) {
        return (exchange, chain) -> {
            try {
                String authHeader = exchange.getRequest().getHeaders().getFirst(HttpHeaders.AUTHORIZATION);
                String token = authHeader != null && authHeader.startsWith("Bearer ")
                        ? authHeader.substring(7)
                        : null;
                String sourceUri = exchange.getRequest().getPath().value();
                String requestId;
                if (exchange.getAttribute(REQUEST_ID) == null) {
                    requestId = UUID.randomUUID().toString();
                    exchange.getAttributes().put(REQUEST_ID, requestId);
                }else {
                    requestId = exchange.getAttributes().get(REQUEST_ID).toString();
                }
                ServerWebExchange modifiedExchange = exchange.mutate()
                        .request(builder -> builder
                                .headers(headers -> {
                                    // 添加新header
                                    headers.add(REQUEST_ID, requestId);
                                    // 修改已有header
                                    headers.putAll(exchange.getRequest().getHeaders());
                                })
                        )
                        .build();
                modifiedExchange.getAttributes().put(REQUEST_ID, requestId);

                if (token != null) {
                    return tokenRouteMappingService.findMatchingUriConfig(token, sourceUri)
                            .map(uriConfig -> cacheRequestBody(modifiedExchange).flatMap(cachedBody -> {
                                log.info("Found matching URI config for token: {}, sourceUri: {}, request id:{}", token, sourceUri, modifiedExchange.getAttributes().get(REQUEST_ID));
                                String primaryUrl = uriConfig.getPrimaryUrl();
                                CircularQueue errorStats = getOrCreateErrorStats(primaryUrl);
                                long startTime = System.currentTimeMillis();
                                // 检查是否应该跳过主地址
                                if (shouldSkipPrimary(uriConfig.getPrimaryUrl())) {
                                    //告警日志
                                    log.info("Directly using fallback due to high error rate");
                                    String targetUri = tokenRouteMappingService.resolveTargetUri(sourceUri, uriConfig);
                                    return tryRequest(
                                            uriConfig.getFallbackUrl(),
                                            targetUri,
                                            uriConfig.getFallbackHost(),
                                            uriConfig.getModel(),
                                            uriConfig.getModelKey(),
                                            modifiedExchange,
                                            false,
                                            null
                                    );
                                }

                                return tryRequest(
                                        uriConfig.getPrimaryUrl(),
                                        sourceUri,
                                        uriConfig.getPrimaryHost(),
                                        uriConfig.getModel(),
                                        uriConfig.getModelKey(),
                                        modifiedExchange,
                                        true,
                                        errorStats
                                ).onErrorResume(throwable -> {
                                            log.error("Primary endpoint failed: {}, error: {}",
                                                    uriConfig.getPrimaryUrl() + sourceUri,
                                                    throwable.getMessage());
                                            long duration = System.currentTimeMillis() - startTime;
                                            log.info("Source request error:{} took:{}ms, switching to target: {} request id:{}",
                                                    throwable.getMessage(), duration, uriConfig.getTargetUri(), modifiedExchange.getAttributes().get(REQUEST_ID));
                                            // 记录错误
                                            getOrCreateErrorStats(uriConfig.getPrimaryUrl()).add(false);

                                            String targetUri = tokenRouteMappingService.resolveTargetUri(sourceUri, uriConfig);
                                            return tryRequest(
                                                    uriConfig.getFallbackUrl(),
                                                    targetUri,
                                                    uriConfig.getFallbackHost(),
                                                    uriConfig.getModel(),
                                                    uriConfig.getModelKey(),
                                                    modifiedExchange,
                                                    false,
                                                    null
                                            );
                                        });
                            }))
                            .orElse(chain.filter(modifiedExchange));
                }

                return chain.filter(modifiedExchange);
            } finally {
                // 确保在请求处理完成后清理 ThreadLocal
                // 清理缓存的请求体
//                exchange.getAttributes().remove(CACHED_REQUEST_BODY_KEY);
            }
        };
    }

    private Mono<String> cacheRequestBody(ServerWebExchange exchange) {

        ServerHttpRequest request = exchange.getRequest();
        // 如果已经缓存过请求体，直接返回
        if (exchange.getAttribute(CACHED_REQUEST_BODY_KEY) != null) {
            return Mono.just("");
        }

        // 对于不需要请求体的方法，直接返回
        if (!requiresRequestBody(request.getMethod())) {
            return Mono.just("");
        }

        return DataBufferUtils.join(request.getBody())
                .map(dataBuffer -> {
                    try {
                        byte[] bytes = new byte[dataBuffer.readableByteCount()];
                        dataBuffer.read(bytes);
                        String body = new String(bytes, StandardCharsets.UTF_8);
                        // 将请求体存储在 ThreadLocal 中
                        log.info("requestId:{},cache body:{}", exchange.getAttributes().get(REQUEST_ID), body);
                        exchange.getAttributes().put(CACHED_REQUEST_BODY_KEY, body);
                        return body;
                    } finally {
                        DataBufferUtils.release(dataBuffer);
                    }
                })
                .defaultIfEmpty("");
    }

    private Mono<Void> tryRequest(String baseUrl, String uri, String host, String model, String modelKey,
                                  ServerWebExchange exchange, boolean isSourceRequest, CircularQueue errorStats) {
        String cachedBody = exchange.getAttribute(CACHED_REQUEST_BODY_KEY);
        ServerHttpRequest request = exchange.getRequest();
        String fullUrl = baseUrl + (baseUrl.endsWith("/") ? uri.substring(1) : uri);

        log.info("request id:{} Starting {} request to: {}", exchange.getAttributes().get(REQUEST_ID), isSourceRequest ? "source" : "target", fullUrl);
        long startTime = System.currentTimeMillis();
        Integer requestTimeout = appConfig.getRequestTimeout();
        // 创建 WebClient 请求
        if (!isSourceRequest) {
            //第二次请求兜底增长时间
            requestTimeout = requestTimeout + 50000;
        }

        WebClient client = webClientBuilder.clone()
                .clientConnector(new ReactorClientHttpConnector(HttpClient.create()
                        .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, requestTimeout)
                        .responseTimeout(Duration.ofMillis(requestTimeout))))
                .build();

        WebClient.RequestBodySpec requestBodySpec = client
                .method(request.getMethod())
                .uri(fullUrl + (request.getQueryParams().isEmpty() ? "" :
                        "?" + request.getQueryParams().toString().substring(1)));

        // 复制请求头
        requestBodySpec.headers(headers -> {
            headers.addAll(request.getHeaders());
            headers.set(HttpHeaders.HOST, host);
            headers.set("model", model);
            headers.set("modelKey", modelKey);
        });

        // 设置请求体
        if (requiresRequestBody(request.getMethod()) && cachedBody != null && !cachedBody.isEmpty()) {
            MediaType contentType = request.getHeaders().getContentType();
            if (contentType != null) {
                requestBodySpec.contentType(contentType);
            }
            requestBodySpec.body(BodyInserters.fromValue(cachedBody));
        }

        return requestBodySpec
                .exchange()
                .timeout(Duration.ofMillis(requestTimeout))
                .flatMap(clientResponse -> handleResponse(exchange, clientResponse, isSourceRequest, errorStats))
                .doFinally(signalType -> {
                    long endTime = System.currentTimeMillis();
                    long duration = endTime - startTime;
                    log.info("{} request completed: {} - took {}ms, signal: {} request id:{}",
                            isSourceRequest ? "Source" : "Target",
                            fullUrl,
                            duration,
                            signalType, exchange.getAttributes().get(REQUEST_ID));
                });
    }

    private Mono<Void> handleResponse(ServerWebExchange exchange,
                                      org.springframework.web.reactive.function.client.ClientResponse clientResponse,
                                      boolean isSourceRequest, CircularQueue errorStats) {
        HttpStatus responseStatus = clientResponse.statusCode();

        if (isSourceRequest) {
            return clientResponse.bodyToMono(String.class)
                    .flatMap(body -> {
                        log.info("Source request response: status={}, body={} request id:{}", responseStatus, body, exchange.getAttributes().get(REQUEST_ID));

                        if (responseStatus != HttpStatus.OK) {
                            // 只在 source 请求时记录错误
                            if (errorStats != null) {
                                errorStats.add(false);
                            }
                            return Mono.error(new RuntimeException(
                                    String.format("Source request HTTP status not 200: %s, body: %s ,request id:%s",
                                            responseStatus.value(), body, exchange.getAttributes().get(REQUEST_ID))));
                        }

                        try {
                            JSONObject jsonBody = JSON.parseObject(body);
                            boolean success = false;
                            String bodyStatus = "";
                            if (jsonBody.containsKey("status")) {
                                Integer status = jsonBody.getInteger("status");
                                bodyStatus = status + "";
                            }
                            if (jsonBody.containsKey("answer")) {
                                success = true;
                            }

                            if (jsonBody.containsKey("data")) {
                                JSONObject data = jsonBody.getJSONObject("data");
                                if (data.containsKey("status")) {
                                    String status = data.getString("status");
                                    if ("succeeded".equalsIgnoreCase(status)) {
                                        success = true;
                                    }
                                    bodyStatus = status;
                                }
                            }
                            if (!success) {
                                // 只在 source 请求时记录错误
                                if (errorStats != null) {
                                    errorStats.add(false);
                                }
                                return Mono.error(new RuntimeException(
                                        String.format("Source request body status not 200: %s, body: %s, request id:%s",
                                                bodyStatus, body, exchange.getAttributes().get(REQUEST_ID))));
                            }
                            // 只在 source 请求时记录错误
                            if (errorStats != null) {
                                errorStats.add(true);
                            }

                            exchange.getResponse().setStatusCode(responseStatus);
                            exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());
                            exchange.getResponse().getHeaders().add(REQUEST_ID, exchange.getAttributes().get(REQUEST_ID).toString());
                            DataBuffer buffer = exchange.getResponse().bufferFactory()
                                    .wrap(body.getBytes(StandardCharsets.UTF_8));
                            return exchange.getResponse().writeWith(Mono.just(buffer));

                        } catch (Exception e) {
                            log.error("Error parsing response body: {} request id:{}", e.getMessage(), exchange.getAttributes().get(REQUEST_ID));
                            return Mono.error(new RuntimeException(
                                    String.format("Failed to parse response body as JSON: %s request id:{}", body, exchange.getAttributes().get(REQUEST_ID))));
                        }
                    });
        }

        exchange.getResponse().setStatusCode(responseStatus);
        exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());

        return clientResponse.bodyToMono(DataBuffer.class)
                .flatMap(body -> {
                    log.info("Target request response: status={}, body={},request id={}", responseStatus, body, exchange.getAttributes().get(REQUEST_ID));
                    DataBuffer body1 = body;
                    // 检查 HTTP 状态码
                    if (!responseStatus.is2xxSuccessful()) {
                        String errorMessage = String.format("Target request failed with status: %s, body: %s,request id={}",
                                responseStatus.value(), body, exchange.getAttributes().get(REQUEST_ID));
                        log.error(errorMessage);
                        return writeErrorResponse(exchange, HttpStatus.BAD_GATEWAY, errorMessage);
                    }

                    try {
                        return writeResponse(exchange, responseStatus, body);
                    } catch (Exception e) {
                        String errorMessage = String.format("Failed to process target response: %s, body: %s,request id={}",
                                e.getMessage(), body, exchange.getAttributes().get(REQUEST_ID));
                        log.error(errorMessage);
                        return writeErrorResponse(exchange, HttpStatus.BAD_GATEWAY, errorMessage);
                    }
                })
                .onErrorResume(throwable -> {
                    log.info("Error handling response: {},request id={}", throwable.getMessage(), exchange.getAttributes().get(REQUEST_ID));
                    exchange.getResponse().setStatusCode(HttpStatus.INTERNAL_SERVER_ERROR);
                    String errorMessage = throwable.getMessage();
                    DataBuffer buffer = exchange.getResponse().bufferFactory()
                            .wrap(errorMessage.getBytes(StandardCharsets.UTF_8));
                    return exchange.getResponse().writeWith(Mono.just(buffer));
                });
    }

    // 辅助方法：写入正常响应
    private Mono<Void> writeResponse(ServerWebExchange exchange, HttpStatus status, DataBuffer buffer) {
        exchange.getResponse().setStatusCode(status);
        return exchange.getResponse().writeWith(Mono.just(buffer));
    }

    // 辅助方法：写入错误响应
    private Mono<Void> writeErrorResponse(ServerWebExchange exchange, HttpStatus status, String message) {
        exchange.getResponse().setStatusCode(status);
        JSONObject errorResponse = new JSONObject();
        errorResponse.put("status", status.value());
        errorResponse.put("message", message);
        DataBuffer buffer = exchange.getResponse().bufferFactory()
                .wrap(errorResponse.toJSONString().getBytes(StandardCharsets.UTF_8));
        return exchange.getResponse().writeWith(Mono.just(buffer));
    }

    private boolean requiresRequestBody(HttpMethod method) {
        return method == HttpMethod.POST ||
                method == HttpMethod.PUT ||
                method == HttpMethod.PATCH;
    }

    @Data
    public static class Config {
    }
}